using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;


namespace Scanner
{
	//
	// The purpose of the Defines class is just to be able to define a string at the top of this file.
	//
	// You must redefine the ScannerDllFilename string for the scanner you intend to use.
	// It must be hardcoded since it's used in the DLLImport attribute below.
	// For the following Sysview scanners, use these filenames:
	//
	//	DocketPORT 487:		DPORT487.DLL
	//	DocketPORT 687:		DPORT667.DLL
	//	DocketPORT 467:		DPORT467.DLL
	//	DocketPORT 667:		DPORT667.DLL
	//
	public class Defines
	{
		public const string ScannerDllFilename = "DPORT487.DLL";
	}


	// Note that some structures are declared as LayoutKind.Explicit and some as LayoutKind.Sequential.
	// The structures declared as explicit are done so to duplicate the C unions that are used by the driver.
	// The structures declared as sequential don't need to be declared explicit because they're not 
	// unions. And they're better declared as sequential so that their alignment matches that of the 
	// driver for 32-bit vs. 64-bit structure packing. The packing level on the driver structures is 
	// the default 8. Structures are not packed the same way for 32-bit and 64-bit drivers. This is 
	// primarily due to the different pointer size. LayoutKind.Sequential takes care of this in each case 
	// below.
	// See http://msdn.microsoft.com/en-us/library/aa366769(VS.85).aspx for detail on structure packing.

	[StructLayout(LayoutKind.Explicit)]
	public struct SIValue
	{
		[FieldOffset(0)]
		public Int32 iVal;
		[FieldOffset(0)]
		public Single fVal;
		[FieldOffset(0)]
		public IntPtr strVal;
		[FieldOffset(0)]
		public Int32 bVal;

		[FieldOffset(0)]
		public IntPtr piVal;
		[FieldOffset(0)]
		public IntPtr psVal;
		[FieldOffset(0)]
		public IntPtr pcVal;
		[FieldOffset(0)]
		public IntPtr puiVal;
		[FieldOffset(0)]
		public IntPtr pusVal;
		[FieldOffset(0)]
		public IntPtr pucVal;
		[FieldOffset(0)]
		public IntPtr pstrVal;
		[FieldOffset(0)]
		public IntPtr pbVal;
		[FieldOffset(0)]
		public IntPtr pfVal;

	};

	[StructLayout(LayoutKind.Sequential, Pack = 8)]
	public struct SISingle
	{
		public SIValue current;
		public SIValue def;
	};

	[StructLayout(LayoutKind.Sequential, Pack = 8)]
	public struct SIRange
	{
		public SIValue minimum;
		public SIValue maximum;
		public SIValue stepSize;
		public SIValue def;
		public SIValue current;
	};

	[StructLayout(LayoutKind.Sequential, Pack=8)]
	public struct SIList
	{
		public Int32 numItems;
		public SIValue current;
		public SIValue def;
		public SIValue items;
	};

	[StructLayout(LayoutKind.Sequential, Pack = 8)]
	public struct SIArray
	{
		public Int32 numItems;
		public SIValue items;
	};

	[StructLayout(LayoutKind.Explicit)]
	public class SIProperty
	{
		[FieldOffset(0)]
		public UInt16 propertyID;
		[FieldOffset(2)]
		public UInt16 containerType;
		[FieldOffset(4)]
		public UInt16 itemType;
		[FieldOffset(6)]
		public UInt16 access;

		// containers
		[FieldOffset(8)]
		public SISingle single;
		[FieldOffset(8)]
		public SIRange range;
		[FieldOffset(8)]
		public SIList list;
		[FieldOffset(8)]
		public SIArray array;


		public const ushort SIP_INVALID = 0;					///< NULL or invalid property
		public const ushort SIP_OPTICAL_RESOLUTION = 2;			///< in pixels per inch
		public const ushort SIP_OPTICAL_WIDTH_IN_PIXELS = 3;
		public const ushort SIP_XRESOLUTION = 4;
		public const ushort SIP_YRESOLUTION = 5;
		public const ushort SIP_BITS_PER_CHANNEL = 6;
		public const ushort SIP_BITS_PER_PIXEL = 7;
		public const ushort SIP_SCAN_MODE = 8;
		public const ushort SIP_BRIGHTNESS = 9;
		public const ushort SIP_CONTRAST = 10;
		public const ushort SIP_PHOTOMETRIC_INTERPRETATION = 11;
		public const ushort SIP_YOFFSET = 12;
		public const ushort SIP_XOFFSET = 13;
		public const ushort SIP_SCAN_WIDTH_IN_PIXELS = 14;
		public const ushort SIP_SCAN_LENGTH_IN_LINES = 15;
		public const ushort SIP_PLANARCHUNKY = 16;
		public const ushort SIP_CHANNEL_ORDER = 17;				///< The order of RGB bytes in a color pixel. valid values are constants that begin with SI_CO_
		public const ushort SIP_MAX_SCAN_TIME_IN_SEC = 18;
		public const ushort SIP_LINE_WIDTH_IN_BYTES = 19;		///< always a read-only value. Depends on scan mode, x offset, width.
		public const ushort SIP_DUPLEX_ENABLED = 21;
		public const ushort SIP_SPOOLER_ENABLED = 22;
		public const ushort SIP_FEED_DIRECTION = 23;
		public const ushort SIP_OPTICAL_LENGTH_IN_PIXELS = 24;
		public const ushort SIP_GAMMA = 25;
		public const ushort SIP_HIGHLIGHT = 26;
		public const ushort SIP_SHADOW = 27;
		public const ushort SIP_LUT_GRAY = 28;
		public const ushort SIP_LUT_RED = 29;
		public const ushort SIP_LUT_GREEN = 30;
		public const ushort SIP_LUT_BLUE = 31;
		public const ushort SIP_THRESHOLD = 32;
		public const ushort SIP_EOP_DETECT_ENABLED = 33;
		public const ushort SIP_DESCREEN_ENABLED = 34;
		public const ushort SIP_DROPOUT_COLOR = 35;
		public const ushort SIP_LED_INDICATOR1 = 40;
		public const ushort SIP_LED_INDICATOR2 = 41;
		public const ushort SIP_USB_RATE = 220;
		public const ushort SIP_FEED_RATE = 1004;
		public const ushort SIP_SCAN_RATE = 1006;


		// Container types
		public const ushort SICON_SINGLE = 0;
		public const ushort SICON_RANGE = 1;
		public const ushort SICON_LIST = 2;
		public const ushort SICON_ARRAY = 3;

		// Item types
		public const ushort SI_INT32 = 1;		///< 32-bit signed integer
		public const ushort SI_UINT32 = 2;		///< 32-bit unsigned integer
		public const ushort SI_INT16 = 3;		///< 16-bit signed integer
		public const ushort SI_UINT16 = 4;		///< 16-bit unsigned integer
		public const ushort SI_INT8 = 5;		///< 8-bit signed integer
		public const ushort SI_UINT8 = 6;		///< 8-bit unsigned integer
		public const ushort SI_BOOL = 7;		///< The item will be either SI_TRUE or SI_FALSE.
		public const ushort SI_FLOAT32 = 8;		///< single-precision floating point 
		public const ushort SI_STR = 9;			///< string. The item will be a pointer to an ANSI zero-terminated string.

		// Access types
		public const ushort SIACC_READONLY = 0;
		public const ushort SIACC_READWRITE = 1;


		public const int SI_SCANMODE_BW = 0;
		public const int SI_SCANMODE_GRAY = 1;
		public const int SI_SCANMODE_RGB = 2;

		public const int SI_PI_BLACKZERO = 0;	///< Photometric Interpretation. Black is represented by zero
		public const int SI_PI_WHITEZERO = 1;	///< Photometric Interpretation. White is represented by zero

		public const int SI_PC_PLANAR = 0;
		public const int SI_PC_CHUNKY = 1;

		public const int SI_CO_RGB = 0;		///< Value of SIP_CHANNEL_ORDER. SI_CO_RGB specifies that pixels are in red-green-blue order.
		public const int SI_CO_BGR = 1;		///< Value of SIP_CHANNEL_ORDER. SI_CO_BRG specifies that pixels are in blue-green-red order.

		public const int SI_DO_NONE = 0;	///< Value of SIP_DROPOUT_COLOR. SI_DO_NONE means no dropout.
		public const int SI_DO_RED = 1;		///< Value of SIP_DROPOUT_COLOR. SI_DO_RED means red dropout.
		public const int SI_DO_GREEN = 2;	///< Value of SIP_DROPOUT_COLOR. SI_DO_GREEN means green dropout.
		public const int SI_DO_BLUE = 3;	///< Value of SIP_DROPOUT_COLOR. SI_DO_BLUE means blue dropout.SI_SCANMODE_BW = 0;

		public const int SI_FALSE = 0;
		public const int SI_TRUE = 1;

		// Motor feed directions. Used for SIP_FEED_DIRECTION.
		public const int SI_FORWARD = 0;
		public const int SI_REVERSE = 1;

		// These identifiers used with the SIP_USB_RATE property.
		public const int SI_USB_FULL = 1;
		public const int SI_USB_HIGH = 2;

	} ;


	[StructLayout(LayoutKind.Sequential, Pack = 8)]
	public struct SIEventInfo
	{
		public byte eventDataSize;			// [in] The size in bytes of the data in the eventData array.

		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 20)]
		public byte[] eventData;			// [in] The data return by the scanner in the USB interrupt phase.
		public uint eventFlags;				// [out] Bit flags that indicate which event(s) occurred.
	};


	public enum SIResult : uint
	{
		SIR_SUCCESS =						0,
		SIR_UNKNOWN_MODEL_NAME =			10,
		SIR_SCANNER_NOT_READY =				11,
		SIR_BAD_PARAMETER =					14,
		SIR_ALREADY_OPEN =					15,
		SIR_INTERFACE_NOT_OPEN =			17,
		SIR_FILESYSTEM_ACCESS_DENIED =		18,
		SIR_SCANNER_BUSY =					19,
		SIR_ENDOFDATA =						20,
		SIR_NOT_SCANNING =					21,
		SIR_DEVICE_COMMUNICATION_ERROR =	0x100,
		SIR_UNKNOWN_ERROR =					0x800,
		SIR_NOT_CALIBRATED =				0x802,
		SIR_INITIALIZATION_FAILURE =		0x803,
		SIR_USER_CANCELLED =				0x804,
		SIR_CALIB_WHITE_TARGET =			0x840,
		SIR_PROPERTY_UNSUPPORTED =			0x900,			///< The caller tried to set or retrieve a capabiity that is not supported in the current implementation.
		SIR_PROPERTY_INVALID_VALUE =		0x901,			///< The caller tried to set a property that was outside of the valid range.
		SIR_PROPERTY_READONLY =				0x902,			///< The caller tried to set a property that has read-only access.
	}

	public enum SICalibrationState : uint
	{
		NOT_CALIBRATED =	0,
		CALIBRATED =		1,
	}

	// Valid parameters for the SI_Calibrate() function.
	// These are types of calibration targets.
	public enum SICalibrationTarget : uint
	{
		SI_CT_DEFAULT =		0,
		SI_CT_WHITEONLY =	1,
	}

	public enum SIScannerStatus : uint
	{
		SI_SS_OFFLINE = 0,
		SI_SS_ONLINE_READY = 1,
		SI_SS_ONLINE_BUSY = 2,
	}

	public enum SIButtonStatus : uint
	{
		SI_BS_UP =			0,
		SI_BS_DOWN =		1,
	}

	public enum SIPaperStatus : uint
	{
		SI_PS_PAPER_OUT =	0,
		SI_PS_PAPER_IN =	1,
	}

	public enum SICallbackResult : uint
	{
		CANCEL =	0,
		CONTINUE =	1,
	}

	[UnmanagedFunctionPointer(CallingConvention.Cdecl)] 
	public delegate SICallbackResult ProgressCallbackDelegate(uint percentCompleted);

	public enum SIDiagTest : uint
	{
		SI_DIAG_HW_PRESENCE =			0x00,
		SI_DIAG_READ_PERFORMANCE =		0x10,
	}

	public enum SIDiagResult : uint
	{
		SI_DIAG_FAIL = 0,
		SI_DIAG_PASS = 1
	};

	public abstract class SIDiagInfo 
	{
	};

	public class SIDiagHwPresenceInfo : SIDiagInfo
	{
		public SIDiagResult result;
	};

	public class SIDiagReadPerfInfo : SIDiagInfo
	{
		public uint readLengthInKB;		//	[in]  Set this to the length of data to read from the hardware.
		public uint timeInMicroSec;		//	[out] Resulting time in microseconds will be returned here.
	};



	class SIScanner
	{

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_OpenInterface",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_OpenInterface(string pName);

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_CloseInterface",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_CloseInterface();


		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_IsCalibrated",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_IsCalibrated(ref SICalibrationState pState);

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_Calibrate",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_Calibrate(SICalibrationTarget target, ProgressCallbackDelegate progressCallback);

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_GetScannerStatus",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_GetScannerStatus(ref SIScannerStatus pScannerStatus);

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_GetPaperStatus",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_GetPaperStatus(ref SIPaperStatus pPaperStatus);

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_GetButtonStatus",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_GetButtonStatus(ref SIButtonStatus pButtonStatus);


		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_GetProperty",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_GetProperty(SIProperty prop);

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_SetProperty",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_SetProperty(SIProperty prop);


		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_StartScan",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_StartScan();

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_ReadImageData",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_ReadImageData(
									byte[] buffer,
									int numberOfLinesToRead,
									int pageNumber,
									ref uint numberOfLinesReturned);

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_EndScan",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_EndScan();

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_Feed",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_Feed();

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_FeedPaperOut",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_FeedPaperOut();

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_Clean",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_Clean(ProgressCallbackDelegate progressCallback);


		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_Reset",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_Reset();

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_Diagnostic",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_Diagnostic(SIDiagTest test, SIDiagInfo diagInfo);


		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_GetLastError",
							SetLastError = true,
							CharSet = CharSet.Ansi,
							ExactSpelling = true,
							CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_GetLastError();

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_GetLastErrorText",
					SetLastError = true,
					CharSet = CharSet.Ansi,
					ExactSpelling = true,
					CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_GetLastErrorText(ref string ppErrorText);

		[DllImport(Defines.ScannerDllFilename, EntryPoint = "SI_GetEvent",
					SetLastError = true,
					CharSet = CharSet.Ansi,
					ExactSpelling = true,
					CallingConvention = CallingConvention.Cdecl)]
		public static extern SIResult SI_GetEvent(ref SIEventInfo eventInfo);

	}
}
