// ChildView.cpp : implementation of the CChildView class
//

#include "stdafx.h"
#include <assert.h>
#include "Miniscan.h"
#include "ChildView.h"

#include "gdiplus.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CChildView

CChildView::CChildView()
	:m_currentViewSide(0),
	m_fitToWindow(true),
	m_leftButtonIsDown(false)
{

	Gdiplus::GdiplusStartupInput gdiplusStartupInput;
	Gdiplus::GdiplusStartup(&m_gdiplusToken, &gdiplusStartupInput, NULL);

	m_frontImage.pBitmap = NULL;
	m_frontImage.pImageBuffer = NULL;

	m_backImage.pBitmap = NULL;
	m_backImage.pImageBuffer = NULL;

}

CChildView::~CChildView()
{

	if(m_frontImage.pBitmap)
	{
		delete m_frontImage.pBitmap;
	}
	if(m_frontImage.pImageBuffer)
	{
		delete m_frontImage.pImageBuffer;
	}

	if(m_backImage.pBitmap)
	{
		delete m_backImage.pBitmap;
	}
	if(m_backImage.pImageBuffer)
	{
		delete m_backImage.pImageBuffer;
	}

	Gdiplus::GdiplusShutdown(m_gdiplusToken);

}


BEGIN_MESSAGE_MAP(CChildView, CScrollView)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_WINDOWPOSCHANGED()
	ON_WM_LBUTTONUP()
	ON_WM_SETCURSOR()
END_MESSAGE_MAP()



// CChildView message handlers

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), reinterpret_cast<HBRUSH>(COLOR_WINDOW+1), NULL);

	if (!CScrollView::PreCreateWindow(cs))
		return FALSE;

	return TRUE;
}

// Call SetBitmapFilename() to specify the filename of the image to display 
// in the client area. When SetBitmapFilename() is called, the file is loaded 
// and displayed.
void CChildView::SetBitmapFilename(int side, CString filename)
{
	assert((side == 0) || (side == 1));

	if(side == 0)
	{
		m_frontImage.filename = filename;
		LoadBitmapFile(m_frontImage);

	}
	else if (side == 1)
	{
		m_backImage.filename = filename;
		LoadBitmapFile(m_backImage);

	}
	
	UpdateScale();
	Invalidate();

}

// Call SetCurrentView() to specify which side of the scanned image should be 
// displayed, front or back.
// 0 = front
// 1 = back
void CChildView::SetCurrentView(int side)
{
	assert((side == 0) || (side == 1));

	m_currentViewSide = side;

	UpdateScale();

	Invalidate();
}

// Call SetZoom() to specify the zoom level at which the image is displayed.
// If fitToWindow = true, the image is scaled to fit into the client area.
// If fitToWindow = false, the image is not scaled at all, it's 1:1.
void CChildView::SetZoom(int scale)
{
	if(scale == 0)
	{
		m_fitToWindow = true;
	}
	else
	{
		m_fitToWindow = false;
		m_scale = (float)scale;
	}


	UpdateScale();

	Invalidate();
}

// UpdateScale() will update the m_scale member variable to indicate the scale that 
// the image should be displayed in, according to the current value of m_fitToWindow.
//
// If m_fitToWindow is false, then the scale will be 1:1.
// If m_fitToWindow is true, then the scale factor will be set so that the image
//    will fit in the client ares of the window.
//
void CChildView::UpdateScale()
{
	Bitmap* pBitmap = (m_currentViewSide == 0) ? m_frontImage.pBitmap : m_backImage.pBitmap;

	if(pBitmap)
	{
		if(!m_fitToWindow)
		{
			//m_scale = 2;

		}
		else
		{

			// Find out how big the client window area is.
			CRect destRect;
			GetClientRect(destRect);

			// scale to fit the client area.
			float xScale = float(destRect.Width()) / pBitmap->GetWidth();
			float yScale = float(destRect.Height()) / pBitmap->GetHeight();
			m_scale = min(xScale, yScale);

		}

		SetScrollSizes(MM_TEXT, CSize(int(m_scale * pBitmap->GetWidth()), 
				int(m_scale * pBitmap->GetHeight())));


	}

}


void CChildView::OnDraw(CDC *pDC) 
{

	Bitmap* pBitmap = (m_currentViewSide == 0) ? m_frontImage.pBitmap : m_backImage.pBitmap;

	if(pBitmap)
	{
		Graphics graphics(pDC->GetSafeHdc());

		graphics.SetInterpolationMode(InterpolationModeNearestNeighbor);

		// Setup the destination rectangle as a RectF, which is what GDI+ needs to see.
		RectF destRectF(
			0,  // m_imageOffset.x,
			0,  // m_imageOffset.y, 
			m_scale * pBitmap->GetWidth(), 
			m_scale * pBitmap->GetHeight());
	
		// draw the scaled image
		graphics.DrawImage(pBitmap, 
			destRectF,		// destination rectangle
			0, 				// x position for upper-left corner of the image.
			0, 				// y position for upper-left corner of the image.
			REAL(pBitmap->GetWidth()),	// x portion of the source image to draw
			REAL(pBitmap->GetHeight()),	// y portion of the source image to draw
			UnitPixel);
	}

}

// Loads a bitmap file into a GDI+ Bitmap object.
// Don't use Bitmap::FromFile() since it locks the file.
void CChildView::LoadBitmapFile(ImageInfo& imageInfo)
{
	if(imageInfo.pBitmap)
	{
		delete imageInfo.pBitmap;
		imageInfo.pBitmap = NULL;
	}
	if(imageInfo.pImageBuffer)
	{
		delete imageInfo.pImageBuffer;
		imageInfo.pImageBuffer = NULL;
	}
	
	if(imageInfo.filename.GetLength() == 0)
	{
		return;
	}

	FILE*	pInFile;
	errno_t res = fopen_s(&pInFile, imageInfo.filename, "rb");

	if(pInFile == NULL)
	{
		return ;
	}

	BITMAPFILEHEADER bmFileHeader;
	fread(&bmFileHeader, sizeof(bmFileHeader), 1, pInFile);

	BITMAPINFO* pbmInfo;

	// calculate the size of the BITMAPINFOHEADER and the RGBQUAD array.
	int sizeOfBitmapInfo = bmFileHeader.bfOffBits - sizeof(bmFileHeader);

	pbmInfo = (BITMAPINFO* )new char[sizeOfBitmapInfo];

	fread(pbmInfo, sizeOfBitmapInfo, 1, pInFile);

	imageInfo.pImageBuffer = new char[pbmInfo->bmiHeader.biSizeImage];

	fread(imageInfo.pImageBuffer, 1, pbmInfo->bmiHeader.biSizeImage, pInFile);

	imageInfo.pBitmap = Bitmap::FromBITMAPINFO(pbmInfo, (VOID *)imageInfo.pImageBuffer);

	fclose(pInFile);
	delete pbmInfo;

}


void CChildView::OnLButtonDown(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default

	m_prevMousePos = point;

	if(!m_fitToWindow)
	{
		SetCursor(::LoadCursor(AfxGetInstanceHandle(), (LPCSTR)IDC_SCROLL_CURSOR));
	}

	m_leftButtonIsDown = true;

	CScrollView::OnLButtonDown(nFlags, point);
}

void CChildView::OnLButtonUp(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default
	m_leftButtonIsDown = false;

	SetCursor(::LoadCursor(NULL, IDC_ARROW));

	CScrollView::OnLButtonUp(nFlags, point);
}

// OnMouseMove() is used to process mouse movements when the left button
// is down. This lets you scroll the image around the window by pressing 
// the button and moving the mouse.
void CChildView::OnMouseMove(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default

	if((nFlags & MK_LBUTTON) && !m_fitToWindow)
	{
		CPoint diff;

		diff = m_prevMousePos - point;
		m_prevMousePos = point;

		CPoint pt = GetScrollPosition();
		pt += diff;

		// Never scroll to a negative offset.
		pt.x = max(0, pt.x);
		pt.y = max(0, pt.y);

		// Only scroll as far as necessary to make sure all of the image 
		// can be seen in the client area, but no further.
		CRect destRect;
		GetClientRect(destRect);

		Bitmap* pBitmap = (m_currentViewSide == 0) ? m_frontImage.pBitmap : m_backImage.pBitmap;
		if(pBitmap)
		{
			pt.x = min(pt.x, int(m_scale * pBitmap->GetWidth()) - destRect.Width());
			pt.y = min(pt.y, int(m_scale * pBitmap->GetHeight()) - destRect.Height());
		}

		ScrollToPosition(pt);


	}

	CScrollView::OnMouseMove(nFlags, point);
}

void CChildView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	// TODO: Add your specialized code here and/or call the base class

	SetScrollSizes(MM_TEXT, CSize(100,100));

}


void CChildView::OnWindowPosChanged(WINDOWPOS* lpwndpos)
{
	CScrollView::OnWindowPosChanged(lpwndpos);

	// The window size may have changed as a result of the user resizing 
	// the window. So recalculate the scale. A WM_PAINT message will come 
	// along later to cause OnDraw() to be called.
	if(m_fitToWindow)
	{
		UpdateScale();
	}
}


// OnSetCursor() is called by MFC when the mouse moves.
// In order to prevent MFC from changing the cursor from the "hand" 
// back to an arrow when the mouse is moved, you need to override this function.
//
BOOL CChildView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
	// If buttong is down and the scale is 1:1, then display the "hand" cursor.
	if(m_leftButtonIsDown && !m_fitToWindow)
	{
		SetCursor(::LoadCursor(AfxGetInstanceHandle(), (LPCSTR)IDC_SCROLL_CURSOR));
		return TRUE; // halt further processing
	}

	return CScrollView::OnSetCursor(pWnd, nHitTest, message);
	
}
