// TiffDump.h
//
#ifndef _TIFF_DUMP_H_
#define _TIFF_DUMP_H_

// ========================================================
//Use CTiffDump to spool an image to a TIFF file. The image is spooled row by row from
//top to bottom. Supports 1-bit B&W, 8-bit gray, 16-bit gray, 24-bit color and 48-bit color
//
//Suggested calling sequence:
//
//	// create and open TIFF file for writing
//	Create();
//
//	// Specify image attributes
//	SetWidthInPixels();		// required
//	SetNumChannels();		// required
//	SetBitsPerChannel();	// required
//	SetResolution();		// optional
//
//	// Get the image width in bytes
//	fileRowBytes = imgFile.GetRowBytes();
//
//	// Write the image data
//	AppendLines();	// one or more calls
//
//	// release memory, close file, etc.
//	Cleanup();		// optional (called by destructor)
//
// ========================================================

#include <stdio.h>
#include "PlatTypes.h"

class CTiffDump
{
public:

	// return codes
	enum {
		kTDNoErr = 0,
		kTDParamErr = 1,
		kTDOpenErr = 2,
		kTDWriteErr = 3
	};

	// Construction / destruction
	CTiffDump();
	~CTiffDump();

	// Create file
	int Create(const char *path);

	// Set file attributes
	void SetWidthInPixels(int32 pw);
	void SetNumChannels(int nc);
	void SetBitsPerChannel(int bpc);
	void SetResolution(int dpi);

	// Get file attributes
	int32 GetRowBytes();

	// Write image data
	int AppendLines(int32 numLines, const void *pLineBuf);

	// Cleanup
	void Cleanup();

private:
	CTiffDump(CTiffDump&) {}; // Copy not supported

	FILE	*fileRef;
	int32	numLines;
	int32	rowBytes;
	int32	pixelWidth;
	int		numChannels;
	int		bitsPerChannel;
	int		resolution;

	void	CloseFile();
	int		WriteHeader();
};

#endif //_TIFF_DUMP_H_
