#pragma once

namespace SINative {
#include "ScannerAPI.h"		// the project needs to point to the inc directory for additional headers
}

using namespace SINative;
using namespace System;


typedef SIResult	(SICALLTYPE* SI_GetScannerStatusProc)(SIScannerStatus *pScannerStatus);
typedef SIResult	(SICALLTYPE* SI_GetPaperStatusProc)(SIPaperStatus *pPaperStatus);
typedef SIResult	(SICALLTYPE* SI_GetButtonStatusProc)(uint32 buttonNumber, SIButtonStatus *pButtonStatus);
typedef SIResult	(SICALLTYPE* SI_GetPropertyProc)(SIProperty* pProperty);
typedef SIResult	(SICALLTYPE* SI_SetPropertyProc)(SIProperty* pProperty);
typedef SIResult	(SICALLTYPE* SI_ResetProc)();
typedef SIResult	(SICALLTYPE* SI_DiagnosticProc)(SIDiagTest test, void* pDiagInfo);
typedef SIResult	(SICALLTYPE* SI_GetLastErrorProc)();
typedef SIResult	(SICALLTYPE* SI_GetLastErrorTextProc)(char **ppErrorText);
typedef SIResult	(SICALLTYPE* SI_StartScanProc)();
typedef SIResult	(SICALLTYPE* SI_ReadImageDataProc)(uint8 *buffer, uint32 numberOfLinesToRead, uint32 pageNumber, uint32* numberOfLinesReturned);
typedef SIResult	(SICALLTYPE* SI_EndScanProc)();
typedef SIResult	(SICALLTYPE* SI_OpenInterfaceProc)(const char* pName);
typedef SIResult	(SICALLTYPE* SI_CloseInterfaceProc)();
typedef SIResult	(SICALLTYPE* SI_GetEventProc)(SIEventInfo* pEventInfo);
typedef SIResult	(SICALLTYPE* SI_CalibrateProc)(uint32 target, ProgressCallbackPtr pProgressCallback);
typedef SIResult	(SICALLTYPE* SI_CleanProc)(ProgressCallbackPtr pProgressCallback);
typedef SIResult	(SICALLTYPE* SI_IsCalibratedProc)(SICalibrationState* calState);
typedef SIResult	(SICALLTYPE* SI_FeedPaperOutProc)();
typedef SIResult	(SICALLTYPE* SI_FeedProc)();


class ScanLog;

ref class Scanner
{

public:
	Scanner();
	~Scanner();

	bool Attach(const char* DllPathname);
	bool IsAttached()	
	{	return (m_hScannerDLL != NULL);	}
	void Detach(void);

	SI_GetScannerStatusProc		SI_GetScannerStatus;
	SI_GetPaperStatusProc		SI_GetPaperStatus;
	SI_GetButtonStatusProc		SI_GetButtonStatus;
	SI_GetPropertyProc			SI_GetProperty;
	SI_SetPropertyProc			SI_SetProperty;
	SI_ResetProc				SI_Reset;
	SI_DiagnosticProc			SI_Diagnostic;
	SI_GetLastErrorProc			SI_GetLastError;
	SI_GetLastErrorTextProc		SI_GetLastErrorText;
	SI_StartScanProc			SI_StartScan;
	SI_ReadImageDataProc		SI_ReadImageData;
	SI_EndScanProc				SI_EndScan;
	SI_OpenInterfaceProc		SI_OpenInterface;
	SI_CloseInterfaceProc		SI_CloseInterface;
	SI_GetEventProc				SI_GetEvent;
	SI_CalibrateProc			SI_Calibrate;
	SI_CleanProc				SI_Clean;
	SI_IsCalibratedProc			SI_IsCalibrated;
	SI_FeedPaperOutProc			SI_FeedPaperOut;
	SI_FeedProc					SI_Feed;

private:
	HMODULE	m_hScannerDLL;
};

