// ScanActionsDialog.cpp : implementation file
//

#include "stdafx.h"
#include "SpeedbirdTest.h"
#include "ScanActionsDialog.h"
#include "assert.h"
#include "MainFrm.h"
#include "Scanner.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// ScanActionsDialog dialog

IMPLEMENT_DYNAMIC(ScanActionsDialog, DialogBarEx)
ScanActionsDialog::ScanActionsDialog(CWnd* pParent /*=NULL*/)
	:m_lineBuffer(NULL),
	m_scanState(SS_IDLE)
{

	m_filename[0] = "front.bmp";
	m_filename[1] = "back.bmp";
}

ScanActionsDialog::~ScanActionsDialog()
{
}

void ScanActionsDialog::DoDataExchange(CDataExchange* pDX)
{
	DialogBarEx::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(ScanActionsDialog, DialogBarEx)
	ON_BN_CLICKED(IDC_START_SCAN_BUTTON, OnBnClickedStartScanButton)
	ON_BN_CLICKED(IDC_END_SCAN_BUTTON, OnBnClickedEndScanButton)
	ON_BN_CLICKED(IDC_FEED_BUTTON, OnBnClickedFeedButton)
	ON_BN_CLICKED(IDC_FEED_PAPER_OUT_BUTTON, OnBnClickedFeedPaperOutButton)
END_MESSAGE_MAP()


// ScanActionsDialog message handlers


BOOL ScanActionsDialog::OnInitDialogBar()
{
	DialogBarEx::OnInitDialogBar();

	// TODO:  Add extra initialization here
	m_lineBuffer = new unsigned char[BUFFER_SIZE_IN_BYTES];

	assert(m_lineBuffer);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void ScanActionsDialog::PostNcDestroy()
{
	// TODO: Add your specialized code here and/or call the base class
	if(m_lineBuffer)
	{
		delete [] m_lineBuffer;
		m_lineBuffer = NULL;
	}

	DialogBarEx::PostNcDestroy();
}


// OnBnClickedStartScanButton() will acquire all the scan parameters needed to 
// read the image data and save it to a file. This info is saved in member variables.
// Then the scan state (m_scanState) is advanded from SS_IDLE to SS_READ to stare 
// the read process going in the background.
void ScanActionsDialog::OnBnClickedStartScanButton()
{
	SIResult result;
	


	// Allocate a line buffer. 
	// First, find out how many bytes per line the scanner driver will return.
	SIProperty prop;
	prop.propertyID = SIP_LINE_WIDTH_IN_BYTES;
	result = g_scanner.SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_INT32));

	m_lineWidthInBytes = prop.single.current.iVal;

	// round up to the nearest 4-byte boundary for Windows bitmap format.
	int lineWidthInBytesAligned = ((m_lineWidthInBytes + 3) / 4) * 4;

	m_linesPerBuffer = BUFFER_SIZE_IN_BYTES / lineWidthInBytesAligned;

	
	// Retieve the actual bitsPerPixel, width, and length so we can set up the 
	// bitmap file parameters.
	prop.propertyID = SIP_BITS_PER_PIXEL;
	result = g_scanner.SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_INT32));
	m_bitsPerPixel = prop.single.current.iVal;

	prop.propertyID = SIP_SCAN_WIDTH_IN_PIXELS;
	result = g_scanner.SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
	m_scanWidthInPixels = prop.range.current.iVal;

	prop.propertyID = SIP_SCAN_LENGTH_IN_LINES;
	result = g_scanner.SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
	m_scanLength = prop.range.current.iVal;


	((CMainFrame *)GetParent())->SetBitmapFilename(0, "");
	((CMainFrame *)GetParent())->SetBitmapFilename(1, "");
	
	m_numSidesToRead = 1;

	// If the duplex property exists and if it is enabled, read image
	// data for the back also.
	prop.propertyID = SIP_DUPLEX_ENABLED;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));
		if(prop.single.current.bVal == SI_TRUE)
		{
			m_numSidesToRead++;
		}
	}
	
	result = g_scanner.SI_StartScan();

	if(result != SIR_SUCCESS)
	{
		char *pErrorText;
		g_scanner.SI_GetLastErrorText(&pErrorText);
		MessageBox(pErrorText);
		return;
	}

	m_scanState = SS_INIT;
	m_currentReadSide = 0;

}


// OnIdle() will proceed through the process of reading image data from the
// scanner in the background. This allows you to click the EndScan button 
// and end the scan at any time.
//
// The m_scanState variable keeps track of the state of the process. 
// A single-sided scan will go through the states: 
//		SS_IDLE -> SS_INIT -> SS_READ -> SS_END -> SS_IDLE
// A double-sided scan will go through the states: 
//		SS_IDLE -> SS_INIT -> SS_READ -> SS_INIT -> SS_READ -> SS_END -> SS_IDLE
//
BOOL ScanActionsDialog::OnIdle(LONG lCount)
{

	if(m_scanState == SS_IDLE)
	{
		return FALSE;
	}
	
	if(m_scanState == SS_INIT)
	{

		// Setup a bitmap file (and possibly a second one for the back) for the thread 
		// to write image data into.
		m_bitmapFile[m_currentReadSide].Open(m_filename[m_currentReadSide]);
		m_bitmapFile[m_currentReadSide].SetBitsPerPixel(m_bitsPerPixel);
		m_bitmapFile[m_currentReadSide].SetWidth(m_scanWidthInPixels);
		m_bitmapFile[m_currentReadSide].SetHeight(-m_scanLength);

		m_totalLinesLeftToRead = m_scanLength;
		m_totalLinesRead = 0;
	
		m_scanState = SS_READ;
	}
	
	if(m_scanState == SS_READ)
	{

		if(m_totalLinesLeftToRead > 0)
		{
			SIResult result;
			uint32 linesReturned;
	
			int linesToRead = min(m_linesPerBuffer, m_totalLinesLeftToRead);

			// Get image data from the scanner
			result = g_scanner.SI_ReadImageData(m_lineBuffer, linesToRead, m_currentReadSide, &linesReturned);

			if(result == SIR_SUCCESS)
			{
				// Align the data to a 4-byte boundary
				int bytesToWrite = AlignInPlace(m_lineBuffer, linesReturned, m_lineWidthInBytes);

				// Write the image data out to the file
				m_bitmapFile[m_currentReadSide].Write((char *)(m_lineBuffer), bytesToWrite);

				m_totalLinesRead += linesReturned;
				m_totalLinesLeftToRead -= linesReturned;

			}
			else
			{
				if(result == SIR_ENDOFDATA)
				{
					// If we get SIR_ENDOFDATA, then one of three things happened:
					//   1. End-of-page detection was enabled and this is the end. There are not more lines.
					//   2. We tried to read more lines that we specified in the scan parameters.
					//      (This shouldn't happen if you only read as many as you specify.)
					//   3. The scanner was disconnected before all the expected data was scanned.
					TRACE2("Received SIR_ENDOFDATA for side %d. totalLinesRead = %d\n", m_currentReadSide, m_totalLinesRead);
					m_scanState = ((m_currentReadSide + 1) < m_numSidesToRead) ? SS_INIT : SS_END;
				}
				else	// an error
				{
					char *pErrorText;
					g_scanner.SI_GetLastErrorText(&pErrorText);
					MessageBox(pErrorText);
				
					m_scanState = SS_END;
				}

				// In any case that is not SIR_SUCCESS, update the bitmap file length 
				// and close the file.
				m_bitmapFile[m_currentReadSide].SetHeight(-m_totalLinesRead);
				m_bitmapFile[m_currentReadSide].Close();

				((CMainFrame *)GetParent())->SetBitmapFilename(m_currentReadSide, m_filename[m_currentReadSide]);

				m_currentReadSide++;	// continue to next side if this is a duplex scan
				m_totalLinesLeftToRead = 0;
				
			}

		}
		else	// done reading all lines
		{
			// At this point we've read all the lines in the image.
			// So update the bitmap file length and close the file.
			m_bitmapFile[m_currentReadSide].SetHeight(-m_totalLinesRead);
			m_bitmapFile[m_currentReadSide].Close();
				
			((CMainFrame *)GetParent())->SetBitmapFilename(m_currentReadSide, m_filename[m_currentReadSide]);
			
			m_currentReadSide++;	// continue to next side if this is a duplex scan
			m_scanState = (m_currentReadSide < m_numSidesToRead) ? SS_INIT : SS_END;

		}
	}

	if(m_scanState == SS_CANCEL)
	{
		m_bitmapFile[m_currentReadSide].SetHeight(-m_totalLinesRead);
		m_bitmapFile[m_currentReadSide].Close();

		((CMainFrame *)GetParent())->SetBitmapFilename(m_currentReadSide, m_filename[0]);

		m_scanState = SS_END;
	}


	if(m_scanState == SS_END)
	{

		// call SI_EndScan
		SIResult result = g_scanner.SI_EndScan();

		if(result != SIR_SUCCESS)
		{
			char *pErrorText;
			g_scanner.SI_GetLastErrorText(&pErrorText);
			MessageBox(pErrorText);
		}

		m_scanState = SS_IDLE;

	}


	return TRUE; // return TRUE to get more processing time
}


void ScanActionsDialog::OnBnClickedEndScanButton()
{
	SIResult result;

	if(m_scanState == SS_IDLE)
	{
		// If the scan state is idle, then the call to SI_EndScan will be
		// out of sequence. But call it anyway for testing. Calling SI_EndScan
		// when a scan is not in progress should not return an error.
		result = g_scanner.SI_EndScan();

		if(result != SIR_SUCCESS)
		{
			char *pErrorText;
			g_scanner.SI_GetLastErrorText(&pErrorText);
			MessageBox(pErrorText);
		}
	}
	else
	{
		// if the state is not idle, then a scan is in progress.
		// So this function works like a cancel.
		m_scanState = SS_CANCEL;
	}

}

// returns the number of bytes in the buffer after alignment.
int ScanActionsDialog::AlignInPlace(unsigned char* pBuffer, int numberOfLines, int bytesPerLine)
{
	if(bytesPerLine % 4)
	{
		int bytesPerLineAligned = ((bytesPerLine + 3) / 4) * 4;

		int padBytes = bytesPerLineAligned - bytesPerLine ;

		unsigned char* pSrc = pBuffer + (bytesPerLine * numberOfLines) - 1;
		unsigned char* pDst = pBuffer + (bytesPerLineAligned * numberOfLines) - 1;

		while (pSrc >= pBuffer)
		{
			pDst -= padBytes;

			for(int i = 0; i < bytesPerLine; i++)
			{
				*pDst-- = *pSrc--;
			}
		}

		return bytesPerLineAligned * numberOfLines;
	}

	return bytesPerLine * numberOfLines;
}


void ScanActionsDialog::OnBnClickedFeedButton()
{
	// TODO: Add your control notification handler code here

	SIResult result;

	result = g_scanner.SI_Feed();

	if(result != SIR_SUCCESS)
	{
		char *pErrorText;
		g_scanner.SI_GetLastErrorText(&pErrorText);
		MessageBox(pErrorText);
	}
}

void ScanActionsDialog::OnBnClickedFeedPaperOutButton()
{
	// TODO: Add your control notification handler code here
	SIResult result;

	BeginWaitCursor();

	result = g_scanner.SI_FeedPaperOut();

	EndWaitCursor();

	if(result != SIR_SUCCESS)
	{
		char *pErrorText;
		g_scanner.SI_GetLastErrorText(&pErrorText);
		MessageBox(pErrorText);
	}
}

