Imports System.Runtime.InteropServices


Namespace Syscan

	'
	' The purpose of the Defines class is just to be able to define a string at the top of this file.
	'
	' You must redefine the ScannerDllFilename string for the scanner you intend to use.
	' It must be hardcoded since it's used in the DLLImport attribute below.
	' For the following Sysview scanners, use these filenames:
	'
	'	DocketPORT 487:		DPORT487.DLL
	'	DocketPORT 687:		DPORT667.DLL
	'	DocketPORT 467:		DPORT467.DLL
	'	DocketPORT 667:		DPORT667.DLL
	'
	Public Class Defines
		Public Const ScannerDllFilename As String = "DPORT487.DLL"
	End Class

	' Note that some structures are declared as LayoutKind.Explicit and some as LayoutKind.Sequential.
	' The structures declared as explicit are done so to duplicate the C unions that are used by the driver.
	' The structures declared as sequential don't need to be declared explicit because they're not 
	' unions. And they're better declared as sequential so that their alignment matches that of the 
	' driver for 32-bit vs. 64-bit structure packing. The packing level on the driver structures is 
	' the default 8. Structures are not packed the same way for 32-bit and 64-bit drivers. This is 
	' primarily due to the different pointer size. LayoutKind.Sequential takes care of this in each case 
	' below.
	' See http://msdn.microsoft.com/en-us/library/aa366769(VS.85).aspx for detail on structure packing.

	<StructLayout(LayoutKind.Explicit)> Public Structure SIValue
		<FieldOffset(0)> Public iVal As Int32
		<FieldOffset(0)> Public fVal As Single
		<FieldOffset(0)> Public strVal As IntPtr
		<FieldOffset(0)> Public bVal As Int32
		<FieldOffset(0)> Public piVal As IntPtr
		<FieldOffset(0)> Public psVal As IntPtr
		<FieldOffset(0)> Public pcVal As IntPtr
		<FieldOffset(0)> Public puiVal As IntPtr
		<FieldOffset(0)> Public pusVal As IntPtr
		<FieldOffset(0)> Public pucVal As IntPtr
		<FieldOffset(0)> Public pstrVal As IntPtr
		<FieldOffset(0)> Public pbVal As IntPtr
		<FieldOffset(0)> Public pfVal As IntPtr
	End Structure

	<StructLayout(LayoutKind.Sequential, Pack:=8)> Public Structure SISingle
		Public current As SIValue
		Public def As SIValue
	End Structure

	<StructLayout(LayoutKind.Sequential, Pack:=8)> Public Structure SIRange
		Public minimum As SIValue
		Public maximum As SIValue
		Public stepSize As SIValue
		Public def As SIValue
		Public current As SIValue
	End Structure

	<StructLayout(LayoutKind.Sequential, Pack:=8)> Public Structure SIList
		Public numItems As Int32
		Public current As SIValue
		Public def As SIValue
		Public items As SIValue
	End Structure

	<StructLayout(LayoutKind.Sequential, Pack:=8)> Public Structure SIArray
		Public numItems As Int32
		Public items As SIValue
	End Structure

	<StructLayout(LayoutKind.Explicit)> Public Class SIProperty
		<FieldOffset(0)> Public propertyID As UInt16
		<FieldOffset(2)> Public containerType As UInt16
		<FieldOffset(4)> Public itemType As UInt16
		<FieldOffset(6)> Public access As UInt16

		' containers
		<FieldOffset(8)> Public sngle As SISingle	' single is changed to sngle in Visual Basic because "single" is a keyword.
		<FieldOffset(8)> Public range As SIRange
		<FieldOffset(8)> Public list As SIList
		<FieldOffset(8)> Public array As SIArray


		Public Const SIP_INVALID As UShort = 0					' NULL or invalid property
		Public Const SIP_OPTICAL_RESOLUTION As UShort = 2		' in pixels per inch
		Public Const SIP_OPTICAL_WIDTH_IN_PIXELS As UShort = 3
		Public Const SIP_XRESOLUTION As UShort = 4
		Public Const SIP_YRESOLUTION As UShort = 5
		Public Const SIP_BITS_PER_CHANNEL As UShort = 6
		Public Const SIP_BITS_PER_PIXEL As UShort = 7
		Public Const SIP_SCAN_MODE As UShort = 8
		Public Const SIP_BRIGHTNESS As UShort = 9
		Public Const SIP_CONTRAST As UShort = 10
		Public Const SIP_PHOTOMETRIC_INTERPRETATION As UShort = 11
		Public Const SIP_YOFFSET As UShort = 12
		Public Const SIP_XOFFSET As UShort = 13
		Public Const SIP_SCAN_WIDTH_IN_PIXELS As UShort = 14
		Public Const SIP_SCAN_LENGTH_IN_LINES As UShort = 15
		Public Const SIP_PLANARCHUNKY As UShort = 16
		Public Const SIP_CHANNEL_ORDER As UShort = 17			' The order of RGB bytes in a color pixel. valid values are constants that begin with SI_CO_
		Public Const SIP_MAX_SCAN_TIME_IN_SEC As UShort = 18
		Public Const SIP_LINE_WIDTH_IN_BYTES As UShort = 19		' always a read-only value. Depends on scan mode, x offset, width.
		Public Const SIP_DUPLEX_ENABLED As UShort = 21
		Public Const SIP_SPOOLER_ENABLED As UShort = 22
		Public Const SIP_FEED_DIRECTION As UShort = 23
		Public Const SIP_OPTICAL_LENGTH_IN_PIXELS As UShort = 24
		Public Const SIP_GAMMA As UShort = 25
		Public Const SIP_HIGHLIGHT As UShort = 26
		Public Const SIP_SHADOW As UShort = 27
		Public Const SIP_LUT_GRAY As UShort = 28
		Public Const SIP_LUT_RED As UShort = 29
		Public Const SIP_LUT_GREEN As UShort = 30
		Public Const SIP_LUT_BLUE As UShort = 31
		Public Const SIP_THRESHOLD As UShort = 32
		Public Const SIP_EOP_DETECT_ENABLED As UShort = 33
		Public Const SIP_DESCREEN_ENABLED As UShort = 34
		Public Const SIP_DROPOUT_COLOR As UShort = 35
		Public Const SIP_LED_INDICATOR1 As UShort = 40
		Public Const SIP_LED_INDICATOR2 As UShort = 41
		Public Const SIP_USB_RATE As UShort = 220
		Public Const SIP_FEED_RATE As UShort = 1004
		Public Const SIP_SCAN_RATE As UShort = 1006


		' Container types
		Public Const SICON_SINGLE As UShort = 0
		Public Const SICON_RANGE As UShort = 1
		Public Const SICON_LIST As UShort = 2
		Public Const SICON_ARRAY As UShort = 3

		' Item types
		Public Const SI_INT32 As UShort = 1		' 32-bit signed integer
		Public Const SI_UINT32 As UShort = 2	' 32-bit unsigned integer
		Public Const SI_INT16 As UShort = 3		' 16-bit signed integer
		Public Const SI_UINT16 As UShort = 4	' 16-bit unsigned integer
		Public Const SI_INT8 As UShort = 5		' 8-bit signed integer
		Public Const SI_UINT8 As UShort = 6		' 8-bit unsigned integer
		Public Const SI_BOOL As UShort = 7		' The item will be either SI_TRUE or SI_FALSE.
		Public Const SI_FLOAT32 As UShort = 8	' single-precision floating point 
		Public Const SI_STR As UShort = 9		' string. The item will be a pointer to an ANSI zero-terminated string.

		' Access types
		Public Const SIACC_READONLY As UShort = 0
		Public Const SIACC_READWRITE As UShort = 1


		Public Const SI_SCANMODE_BW As Integer = 0
		Public Const SI_SCANMODE_GRAY As Integer = 1
		Public Const SI_SCANMODE_RGB As Integer = 2

		Public Const SI_PI_BLACKZERO As Integer = 0	' Photometric Interpretation. Black is represented by zero
		Public Const SI_PI_WHITEZERO As Integer = 1	' Photometric Interpretation. White is represented by zero

		Public Const SI_PC_PLANAR As Integer = 0
		Public Const SI_PC_CHUNKY As Integer = 1

		Public Const SI_CO_RGB As Integer = 0		' Value of SIP_CHANNEL_ORDER. SI_CO_RGB specifies that pixels are in red-green-blue order.
		Public Const SI_CO_BGR As Integer = 1		' Value of SIP_CHANNEL_ORDER. SI_CO_BRG specifies that pixels are in blue-green-red order.

		Public Const SI_DO_NONE As Integer = 0		' Value of SIP_DROPOUT_COLOR. SI_DO_NONE means no dropout.
		Public Const SI_DO_RED As Integer = 1		' Value of SIP_DROPOUT_COLOR. SI_DO_RED means red dropout.
		Public Const SI_DO_GREEN As Integer = 2		' Value of SIP_DROPOUT_COLOR. SI_DO_GREEN means green dropout.
		Public Const SI_DO_BLUE As Integer = 3		' Value of SIP_DROPOUT_COLOR. SI_DO_BLUE means blue dropout.SI_SCANMODE_BW = 0;

		Public Const SI_FALSE As Integer = 0
		Public Const SI_TRUE As Integer = 1

		' Motor feed directions. Used for SIP_FEED_DIRECTION.
		Public Const SI_FORWARD As Integer = 0
		Public Const SI_REVERSE As Integer = 1

		' These identifiers used with the SIP_USB_RATE property.
		Public Const SI_USB_FULL As Integer = 1
		Public Const SI_USB_HIGH As Integer = 2
	End Class

	<StructLayout(LayoutKind.Sequential, Pack:=8)> Public Structure SIEventInfo

		Public eventDataSize As Byte		' [in] The size in bytes of the data in the eventData array.

		<MarshalAs(UnmanagedType.ByValArray, SizeConst:=20)> _
		Public eventData As Byte()			' [in] The data return by the scanner in the USB interrupt phase.
		Public eventFlags As UInteger		' [out] Bit flags that indicate which event(s) occurred.
	End Structure


	Public Enum SIResult As UInteger

		SIR_SUCCESS = 0
		SIR_UNKNOWN_MODEL_NAME = 10
		SIR_SCANNER_NOT_READY = 11
		SIR_BAD_PARAMETER = 14
		SIR_ALREADY_OPEN = 15
		SIR_INTERFACE_NOT_OPEN = 17
		SIR_FILESYSTEM_ACCESS_DENIED = 18
		SIR_SCANNER_BUSY = 19
		SIR_ENDOFDATA = 20
		SIR_NOT_SCANNING = 21
		SIR_DEVICE_COMMUNICATION_ERROR = &H100
		SIR_UNKNOWN_ERROR = &H800
		SIR_NOT_CALIBRATED = &H802
		SIR_INITIALIZATION_FAILURE = &H803
		SIR_USER_CANCELLED = &H804
		SIR_CALIB_WHITE_TARGET = &H840
		SIR_PROPERTY_UNSUPPORTED = &H900		' The caller tried to set or retrieve a capabiity that is not supported in the current implementation.
		SIR_PROPERTY_INVALID_VALUE = &H901		' The caller tried to set a property that was outside of the valid range.
		SIR_PROPERTY_READONLY = &H902			' The caller tried to set a property that has read-only access.
	End Enum

	Public Enum SICalibrationState As UInteger

		NOT_CALIBRATED = 0
		CALIBRATED = 1
	End Enum

	' Valid parameters for the SI_Calibrate() function.
	' These are types of calibration targets.
	Public Enum SICalibrationTarget As UInteger
		SI_CT_DEFAULT = 0
		SI_CT_WHITEONLY = 1
	End Enum

	Public Enum SIScannerStatus As UInteger
		SI_SS_OFFLINE = 0
		SI_SS_ONLINE_READY = 1
		SI_SS_ONLINE_BUSY = 2
	End Enum

	Public Enum SIButtonStatus As UInteger
		SI_BS_UP = 0
		SI_BS_DOWN = 1
	End Enum

	Public Enum SIPaperStatus As UInteger
		SI_PS_PAPER_OUT = 0
		SI_PS_PAPER_IN = 1
	End Enum

	Public Enum SICallbackResult As UInteger
		CR_CANCEL = 0
		CR_CONTINUE = 1		' Changed from CONTINUE to CR_CONTINUE since "CONTINUE" is a keyword in Visual Basic.
	End Enum

	<UnmanagedFunctionPointer(CallingConvention.Cdecl)> _
	Public Delegate Function ProgressCallbackDelegate(ByVal percentCompleted As UInteger) As SICallbackResult

	Public Enum SIDiagTest As UInteger
		SI_DIAG_HW_PRESENCE = &H0
		SI_DIAG_READ_PERFORMANCE = &H10
	End Enum

	Public Enum SIDiagResult As UInteger
		SI_DIAG_FAIL = 0
		SI_DIAG_PASS = 1
	End Enum

	Public MustInherit Class SIDiagInfo
	End Class

	Public Class SIDiagHwPresenceInfo : Inherits SIDiagInfo
		Public result As SIDiagResult
	End Class

	Public Class SIDiagReadPerfInfo : Inherits SIDiagInfo
		Public readLengthInKB As UInteger			'	[in]  Set this to the length of data to read from the hardware.
		Public timeInMicroSec As UInteger		'	[out] Resulting time in microseconds will be returned here.
	End Class


	Public Class SIScanner

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_OpenInterface", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_OpenInterface(ByVal pName As String) As SIResult
		End Function


		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_CloseInterface", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_CloseInterface() As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_IsCalibrated", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_IsCalibrated(ByRef pState As SICalibrationState) As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_Calibrate", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_Calibrate(ByVal target As SICalibrationTarget, ByVal progressCallback As ProgressCallbackDelegate) As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_GetScannerStatus", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_GetScannerStatus(ByRef pScannerStatus As SIScannerStatus) As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_GetPaperStatus", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_GetPaperStatus(ByRef pPaperStatus As SIPaperStatus) As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_GetButtonStatus", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_GetButtonStatus(ByRef pButtonStatus As SIButtonStatus) As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_GetProperty", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_GetProperty(ByVal prop As SIProperty) As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_SetProperty", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_SetProperty(ByVal prop As SIProperty) As SIResult
		End Function


		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_StartScan", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_StartScan() As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_ReadImageData", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_ReadImageData( _
		  ByVal buffer As Byte(), _
		  ByVal numberOfLinesToRead As Integer, _
		  ByVal pageNumber As Integer, _
		  ByRef numberOfLinesReturned As UInteger) As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_EndScan", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_EndScan() As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_Feed", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_Feed() As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_FeedPaperOut", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_FeedPaperOut() As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_Clean", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_Clean(ByVal progressCallback As ProgressCallbackDelegate) As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_Reset", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_Reset() As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_Diagnostic", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_Diagnostic(ByVal test As SIDiagTest, ByVal diagInfo As SIDiagInfo) As SIResult
		End Function


		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_GetLastError", _
		  SetLastError:=True, _
		  CharSet:=CharSet.Ansi, _
		  ExactSpelling:=True, _
		  CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_GetLastError() As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_GetLastErrorText", _
		   SetLastError:=True, _
		   CharSet:=CharSet.Ansi, _
		   ExactSpelling:=True, _
		   CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_GetLastErrorText(ByRef ppErrorText As String) As SIResult
		End Function

		<DllImport(Defines.ScannerDllFilename, EntryPoint:="SI_GetEvent", _
		   SetLastError:=True, _
		   CharSet:=CharSet.Ansi, _
		   ExactSpelling:=True, _
		   CallingConvention:=CallingConvention.Cdecl)> _
		Public Shared Function SI_GetEvent(ByRef eventInfo As SIEventInfo) As SIResult
		End Function

	End Class
End Namespace



