// PlatTypes.h
//
// Types used in common interface for all supported platforms

#ifndef PLAT_TYPES_H
#define PLAT_TYPES_H

/////////////////////////////////////////////////////////////////
// Common - Common to all platforms
//
// If a type needs to be defined differently on any platform,
// remove it from this section and define it in each
// platform-specific section below.

// DO NOT USE built-in short, long, float or double types in
// cross-platform sources because their properties (e.g. size)
// are compiler-dependant. The build-in char and int types
// should only be used when their size does not matter. The
// build-in void and bool types are okay to use.

typedef unsigned char uint8;
typedef unsigned short uint16;
//typedef unsigned long uint32; // defined below

typedef signed char int8;
typedef signed short int16;
//typedef signed long int32; // defined below

typedef float	float32;
typedef double	float64;


/////////////////////////////////////////////////////////////////
// Windows - Windows/Intel specific

#if defined(_WIN32) || defined(WIN32) || defined(_WINDOWS)

typedef unsigned long uint32;
typedef signed long int32;

#define PACK8 pack(push, 8)
#define UNPACK pack(pop)

#if !defined(SICALLBACK)
// SI prefix used so as not to interfere with any existing Windows definition.
#define SICALLBACK	__cdecl
#endif

#if !defined(SICALLTYPE)
#define SICALLTYPE	__cdecl
#endif

#if !defined(SCANNERDLL_API)
	// On Windows/Intel platform, the following ifdef block creates macros which
	// simplify exporting from this DLL. 
#ifdef SCANNERDLL_EXPORTS
#define SCANNERDLL_API __declspec(dllexport)
#else
#define SCANNERDLL_API __declspec(dllimport)  
#endif
#endif

/////////////////////////////////////////////////////////////////
// Macintosh - MacOS X specific

#elif defined(macintosh)

typedef unsigned int uint32; // An int because a long is 64 bits on LP64
typedef signed int int32; // An int because a long is 64 bits on LP64

#define PACK8 pack(push, 8)
#define UNPACK pack(pop)

#if !defined(SICALLBACK)
#define SICALLBACK
#endif

#if !defined(SICALLTYPE)
#define SICALLTYPE
#endif

#if !defined(SCANNERDLL_API)
#define SCANNERDLL_API
#endif

/////////////////////////////////////////////////////////////////
// Linux - Linux specific

#elif defined(LINUX)

typedef unsigned int uint32; // An int because a long is 64 bits on LP64
typedef signed int int32; // An int because a long is 64 bits on LP64

#define PACK8 pack(push, 8)
#define UNPACK pack(pop)

#if !defined(SICALLBACK)
#define SICALLBACK
#endif

#if !defined(SICALLTYPE)
#define SICALLTYPE
#endif

#if !defined(SCANNERDLL_API)
#define SCANNERDLL_API
#endif

/////////////////////////////////////////////////////////////////

#else
#error Platform not defined
#endif

#endif //PLAT_TYPES_H

